@@ -69,6 +69,8 @@ formatting = {
             'groupindex': object_properties.GROUP_INDEX_SPEECH,
             'clickable': object_properties.STATE_CLICKABLE,
             'haslongdesc': object_properties.STATE_HAS_LONGDESC,
+            'hasdetails': object_properties.RELATION_HAS_DETAILS,
+            'detailsfor': object_properties.RELATION_DETAILS_FOR
         },
         'braille': {
             'eol': object_properties.EOL_INDICATOR_BRAILLE,
@@ -109,25 +111,28 @@ formatting = {
 
     'speech': {
         'prefix': {
-            'focused': '[]',
+            'ancestor': '[]',
+            'focused': 'detailsFor',
             'unfocused': 'oldAncestors + newAncestors',
             'basicWhereAmI': 'toolbar',
             'detailedWhereAmI' : '[]'
             },
         'suffix': {
+            'ancestor': '[]',
             'focused': '[]',
-            'unfocused': 'newNodeLevel + unselectedCell + clickable + hasLongDesc + ' + TUTORIAL + ' + description',
-            'basicWhereAmI': TUTORIAL + ' + clickable + hasLongDesc + description',
-            'detailedWhereAmI': TUTORIAL + ' + clickable + hasLongDesc + description'
+            'unfocused': 'newNodeLevel + unselectedCell + clickable + pause + hasLongDesc + hasDetails + detailsFor +' + TUTORIAL + ' + description + pause + hasPopup',
+            'basicWhereAmI': TUTORIAL + ' + clickable + hasLongDesc + description + pause + hasPopup + pause + detailsFor + pause + allDetails',
+            'detailedWhereAmI': TUTORIAL + ' + clickable + hasLongDesc + description + pause + hasPopup + detailsFor + pause + allDetails'
             },
         'default': {
+            'ancestor': '[]',
             'focused': '[]',
             'unfocused': 'labelOrName + roleName + availability + ' + MNEMONIC + ' + accelerator + childWidget',
             'basicWhereAmI': 'labelOrName + roleName',
             'detailedWhereAmI' : 'pageSummary'
             },
         pyatspi.ROLE_ALERT: {
-            'unfocused': 'labelOrName + roleName + pause + (expandedEOCs or unrelatedLabels or description)'
+            'unfocused': 'roleName + labelOrName + pause + alertText'
             },
         pyatspi.ROLE_ANIMATION: {
             'unfocused': 'labelAndName'
@@ -151,8 +156,8 @@ formatting = {
             },
         pyatspi.ROLE_CHECK_BOX: {
             'focused': 'checkedState',
-            'unfocused': 'labelOrName + roleName + checkedState + required + pause + invalid + availability + ' + MNEMONIC + ' + accelerator',
-            'basicWhereAmI': 'namedContainingPanel + labelOrName + roleName + checkedState + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
+            'unfocused': 'labelOrName + readOnly + roleName + checkedState + required + pause + invalid + availability + ' + MNEMONIC + ' + accelerator',
+            'basicWhereAmI': 'namedContainingPanel + labelOrName + readOnly + roleName + checkedState + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
             },
         pyatspi.ROLE_CHECK_MENU_ITEM: {
             'focused': 'checkedState',
@@ -165,22 +170,48 @@ formatting = {
             'basicWhereAmI': 'label + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
             },
         pyatspi.ROLE_COLUMN_HEADER: {
-            'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
+            'focused': 'labelAndName + roleName + pause + sortOrder',
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + sortOrder'
             },
         pyatspi.ROLE_COMBO_BOX: {
-            'focused': 'labelAndName + roleName + expandableState',
-            'unfocused': 'labelAndName + roleName + pause + (currentLineText + anyTextSelection or positionInList) + ' + MNEMONIC + ' + accelerator',
-            'basicWhereAmI': 'label + roleName + pause + name + (currentLineText + anyTextSelection or positionInList) + ' + MNEMONIC + ' + accelerator'
+            'focused': 'labelOrName + roleName + expandableState',
+            'unfocused': 'labelOrName + roleName + pause + value + pause + positionInList + ' + MNEMONIC + ' + accelerator',
             },
         pyatspi.ROLE_COMMENT: {
             'focused': 'labelOrName + roleName',
             'unfocused': 'labelOrName + roleName + pause + currentLineText + allTextSelection',
             },
+        # TODO - JD: When we bump dependencies to 2.34, remove this fake role and use the real one.
+        'ROLE_CONTENT_DELETION': {
+            'focused': 'leaving or deletionStart',
+            'unfocused': 'deletionStart + pause + displayedText + pause + deletionEnd',
+            },
+        'ROLE_CONTENT_ERROR': {
+            'unfocused': 'displayedText + pause + invalid',
+            },
+        # TODO - JD: When we bump dependencies to 2.34, remove this fake role and use the real one.
+        'ROLE_CONTENT_INSERTION': {
+            'focused': 'leaving or insertionStart',
+            'unfocused': 'insertionStart + pause + displayedText + pause + insertionEnd',
+            },
+        # TODO - JD: When we bump dependencies to 2.36, remove this fake role and use the real one.
+        'ROLE_CONTENT_MARK': {
+            'focused': 'leaving or markStart',
+            'unfocused': 'markStart + pause + displayedText + pause + markEnd',
+            },
+        # TODO - JD: When we bump dependencies to 2.36, remove this fake role and use the real one.
+        'ROLE_CONTENT_SUGGESTION': {
+            'focused': 'leaving or roleName',
+            },
+        pyatspi.ROLE_DESCRIPTION_LIST: {
+            'focused' : 'leaving or (labelOrName + pause + (numberOfChildren or roleName) + pause + nestingLevel)',
+            'unfocused': 'labelOrName + pause + focusedItem + pause + multiselectableState + (numberOfChildren or roleName) + pause'
+            },
         pyatspi.ROLE_DESCRIPTION_TERM: {
-            'unfocused': '(labelOrName or (displayedText + allTextSelection))',
+            'unfocused': '(labelOrName or (displayedText + allTextSelection) + roleName)',
             },
         pyatspi.ROLE_DESCRIPTION_VALUE: {
-            'unfocused': '(labelOrName or (displayedText + allTextSelection))',
+            'unfocused': '(labelOrName or (displayedText + allTextSelection) + roleName)',
             },
         pyatspi.ROLE_DIAL: {
             'focused': 'value',
@@ -188,18 +219,18 @@ formatting = {
             'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
             },
         pyatspi.ROLE_DIALOG: {
-            'focused': 'labelOrName + roleName + description',
-            'unfocused': 'expandedEOCs or (labelOrName + (unrelatedLabels or description))'
+            'focused': 'labelOrName + roleName + (unrelatedLabels or description)',
+            'unfocused': '(expandedEOCs or (labelOrName + roleName + (unrelatedLabels or description)))'
             },
         pyatspi.ROLE_DOCUMENT_FRAME: {
-            'unfocused': 'label + readOnly + textRole + currentLineText + anyTextSelection + ' + MNEMONIC,
-            'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
-            'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
+            'unfocused': 'labelOrName + readOnly + textRole + currentLineText + anyTextSelection + ' + MNEMONIC,
+            'basicWhereAmI': 'labelOrName + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
+            'detailedWhereAmI': 'labelOrName + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
             },
         pyatspi.ROLE_DOCUMENT_WEB: {
-            'unfocused': 'label + readOnly + textRole + currentLineText + anyTextSelection + ' + MNEMONIC,
-            'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
-            'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
+            'unfocused': 'labelOrName + readOnly + textRole + currentLineText + anyTextSelection + ' + MNEMONIC,
+            'basicWhereAmI': 'labelOrName + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
+            'detailedWhereAmI': 'labelorName + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
             },
         'ROLE_DPUB_LANDMARK': {
             'focused': 'leaving or labelOrName',
@@ -219,7 +250,7 @@ formatting = {
             'basicWhereAmI': 'labelOrName + readOnly + textRole + (textContent or placeholderText) + anyTextSelection + required + pause + invalid + ' + MNEMONIC,
             'detailedWhereAmI': 'labelOrName + readOnly + textRole + (textContentWithAttributes or placeholderText) + anyTextSelection + required + pause + invalid + ' + MNEMONIC,
             },
-        'ROLE_FOOTNOTE': {
+        pyatspi.ROLE_FOOTNOTE: {
             'unfocused': 'labelOrName + roleName + pause + currentLineText + allTextSelection',
             },
         pyatspi.ROLE_FOOTER: {
@@ -230,7 +261,7 @@ formatting = {
             'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
             },
         pyatspi.ROLE_FRAME: {
-            'focused': 'labelOrName + roleName',
+            'focused': 'labelOrName',
             'unfocused': 'labelOrName + roleName + unfocusedDialogCount + availability'
             },
         pyatspi.ROLE_HEADER: {
@@ -255,9 +286,7 @@ formatting = {
             'unfocused': 'labelAndName + unrelatedLabels'
             },
         pyatspi.ROLE_LABEL: {
-            'focused': 'label + displayedText + allTextSelection + roleName',
-            'unfocused': 'label + displayedText + allTextSelection + roleName',
-            'basicWhereAmI': 'label + displayedText + allTextSelection + roleName'
+            'unfocused': 'label + ((displayedText + allTextSelection) or name) + roleName',
             },
         pyatspi.ROLE_LANDMARK: {
             'focused': 'leaving or (roleName + labelAndName)',
@@ -274,17 +303,17 @@ formatting = {
             'basicWhereAmI': 'linkInfo + pause + siteDescription + pause + fileSize + pause + ' + MNEMONIC
             },
         pyatspi.ROLE_LIST: {
-            'focused' : 'leaving or (labelOrName + pause + numberOfChildren + pause + nestingLevel)',
-            'unfocused': 'labelOrName + pause + focusedItem + pause + multiselectableState + numberOfChildren + pause'
+            'focused' : 'leaving or (labelOrName + pause + (numberOfChildren or roleName) + pause + nestingLevel)',
+            'unfocused': 'labelOrName + pause + focusedItem + pause + multiselectableState + (numberOfChildren or roleName) + pause'
             },
         pyatspi.ROLE_LIST_BOX: {
-            'focused': 'labelOrName + multiselectableState + numberOfChildren',
-            'unfocused': 'labelOrName + pause + focusedItem + pause + multiselectableState + numberOfChildren + pause'
+            'focused': 'labelOrName + multiselectableState + (numberOfChildren or roleName)',
+            'unfocused': 'labelOrName + pause + focusedItem + pause + multiselectableState + (numberOfChildren or roleName) + pause'
             },
         pyatspi.ROLE_LIST_ITEM: {
             'focused': 'expandableState',
-            'unfocused': '(labelOrName or (listItemMarker + displayedText + allTextSelection)) + pause + expandableState + pause + positionInList + pause + listBoxItemWidgets',
-            'basicWhereAmI': 'label + roleName + pause + (name or (listItemMarker + displayedText)) + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel) + pause'
+            'unfocused': '(labelOrName or (displayedText + allTextSelection)) + checkedStateIfCheckable + pause + unselectedStateIfSelectable + pause + expandableState + pause + positionInList + pause + listBoxItemWidgets',
+            'basicWhereAmI': 'label + roleName + pause + (name or displayedText) + checkedStateIfCheckable + pause + unselectedStateIfSelectable + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel) + pause'
             },
         pyatspi.ROLE_MATH: {
             'unfocused': 'math',
@@ -330,11 +359,11 @@ formatting = {
             },
         pyatspi.ROLE_MENU_ITEM: {
             'focused': 'expandableState',
-            'unfocused': 'labelOrName + menuItemCheckedState + expandableState + availability + ' + MNEMONIC + ' + accelerator + pause + positionInList',
-            'basicWhereAmI': 'ancestors + pause + labelOrName + pause + accelerator + pause + positionInList + ' + MNEMONIC
+            'unfocused': 'labelOrName + checkedStateIfCheckable + expandableState + availability + ' + MNEMONIC + ' + accelerator + pause + positionInList',
+            'basicWhereAmI': 'ancestors + pause + labelOrName + checkedStateIfCheckable + pause + accelerator + pause + positionInList + ' + MNEMONIC
             },
         pyatspi.ROLE_NOTIFICATION: {
-            'unfocused': 'roleName + unrelatedLabels'
+            'unfocused': 'roleName + labelOrName + pause + (expandedEOCs or unrelatedLabels or description)'
             },
         pyatspi.ROLE_PAGE: {
             'focused': 'label + readOnly + currentLineText + anyTextSelection',
@@ -364,8 +393,8 @@ formatting = {
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
             },
         pyatspi.ROLE_PROGRESS_BAR: {
-            'focused': 'progressBarIndex + progressBarValue',
-            'unfocused': 'progressBarIndex + labelAndName + progressBarValue'
+            'focused': 'progressBarIndex + (progressBarValue or roleName)',
+            'unfocused': 'progressBarIndex + labelAndName + (progressBarValue or roleName)'
             },
         pyatspi.ROLE_PUSH_BUTTON: {
             'focused': 'expandableState',
@@ -374,7 +403,7 @@ formatting = {
             },
         pyatspi.ROLE_RADIO_BUTTON: {
             'focused': 'radioState',
-            'unfocused': '(name or label) + pause + radioState + roleName + availability + lineBreak + ' + MNEMONIC + ' + accelerator + pause + positionInList + pause',
+            'unfocused': 'newRadioButtonGroup + pause + (name or label) + pause + radioState + roleName + availability + lineBreak + ' + MNEMONIC + ' + accelerator + pause + positionInList + pause',
             'basicWhereAmI': 'radioButtonGroup + pause + (name or label) + roleName + pause + radioState + pause + positionInGroup + ' + MNEMONIC + ' + accelerator'
             },
         pyatspi.ROLE_RADIO_MENU_ITEM: {
@@ -389,14 +418,22 @@ formatting = {
             'unfocused': 'labelOrName + radioState + roleName + availability + ' + MNEMONIC + ' + accelerator + positionInList',
             'basicWhereAmI': 'ancestors + labelOrName + roleName + radioState + accelerator + positionInList + ' + MNEMONIC
             },
+        'ROLE_REGION': {
+            'focused': 'leaving or (roleName + labelOrName)',
+            'unfocused': 'labelOrName + roleName + currentLineText + allTextSelection'
+            },
         pyatspi.ROLE_ROW_HEADER: {
-            'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
+            'focused': 'labelAndName + roleName + pause + sortOrder',
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + sortOrder'
             },
         pyatspi.ROLE_SCROLL_BAR: {
             'focused': 'value',
             'unfocused': 'labelOrName + roleName + value + required + availability + ' + MNEMONIC,
             'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
             },
+        pyatspi.ROLE_SCROLL_PANE: {
+            'unfocused': '(currentLineText + allTextSelection) or (labelOrName + roleName)',
+            },
         pyatspi.ROLE_SECTION: {
             'focused': '(labelOrName or (currentLineText + allTextSelection)) + roleName',
             'unfocused': '(labelOrName or (currentLineText + allTextSelection)) + roleName + ' + MNEMONIC,
@@ -407,9 +444,13 @@ formatting = {
             'basicWhereAmI': 'labelOrName + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
             },
         pyatspi.ROLE_SPIN_BUTTON: {
-            'focused': 'name',
-            'unfocused': 'labelAndName + allTextSelection + roleName + required + pause + invalid + availability + ' + MNEMONIC,
-            'basicWhereAmI': 'label + roleName + name + allTextSelection + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
+            'focused': '(displayedText or value)',
+            'unfocused': 'labelAndName + (displayedText or value) + roleName + required + pause + invalid + availability + ' + MNEMONIC,
+            'basicWhereAmI': 'label + roleName + name + (displayedText or value) + ' + MNEMONIC + ' + accelerator + required + pause + invalid'
+            },
+        pyatspi.ROLE_SEPARATOR: {
+            'focused': 'roleName',
+            'unfocused': 'roleName + (labelOrName or displayedText or value) + ' + MNEMONIC,
             },
         pyatspi.ROLE_SPLIT_PANE: {
             'focused': 'value',
@@ -418,19 +459,29 @@ formatting = {
             },
         pyatspi.ROLE_STATIC: {
             'unfocused': '(displayedText or name) + roleName',
-        },
+            },
+        pyatspi.ROLE_STATUS_BAR: {
+            'focused': 'labelAndName + roleName',
+            'unfocused': 'labelAndName + roleName + pause + statusBar',
+            },
+        pyatspi.ROLE_SUBSCRIPT: {
+            'unfocused': 'roleName + currentLineText + allTextSelection',
+            },
+        pyatspi.ROLE_SUPERSCRIPT: {
+            'unfocused': 'roleName + currentLineText + allTextSelection',
+            },
         'ROLE_SWITCH': {
             'focused': 'switchState',
             'unfocused': 'labelOrName + roleName + switchState + availability + ' + MNEMONIC + ' + accelerator',
             'basicWhereAmI': 'labelOrName + roleName + switchState'
             },
-
         pyatspi.ROLE_TABLE: {
             'focused': 'leaving or (labelAndName + pause + table)',
             'unfocused': 'labelAndName + pause + table',
             'basicWhereAmI': 'labelAndName + pause + table'
             },
         pyatspi.ROLE_TABLE_CELL: {
+            'ancestor': 'newRowHeader + newColumnHeader + pause + newRow + pause + newColumn',
             'focused': '((tableCell2ChildLabel + tableCell2ChildToggle) or cellCheckedState) + pause + (expandableState and (expandableState + pause + numberOfChildren + pause))',
             'unfocused': 'tableCellRow + pause',
             'basicWhereAmI': 'parentRoleName + pause + columnHeader + pause + rowHeader + pause + roleName + pause + cellCheckedState + pause + (realActiveDescendantDisplayedText or imageDescription + image) + pause + columnAndRow + pause + expandableState + pause + nodeLevel + pause',
@@ -482,8 +533,13 @@ formatting = {
             'unfocused': 'labelAndName + roleName',
             },
         pyatspi.ROLE_TOOL_TIP: {
-            'unfocused': 'labelAndName',
-            'basicWhereAmI': 'labelAndName'
+            'focused': 'leaving or roleName',
+            'unfocused': 'roleName + labelAndName',
+            'basicWhereAmI': 'roleName + labelAndName'
+            },
+        pyatspi.ROLE_TREE: {
+            'focused': 'labelAndName + roleName',
+            'unfocused': 'labelAndName + roleName',
             },
         pyatspi.ROLE_TREE_ITEM: {
             'focused': 'expandableState',
@@ -553,6 +609,9 @@ formatting = {
                              + (childWidget and ([Region(" ")] + childWidget))))'
             },
         #pyatspi.ROLE_ARROW: 'default'
+        pyatspi.ROLE_BLOCK_QUOTE: {
+            'unfocused': BRAILLE_TEXT + ' + (roleName and [Region(" " + asString(roleName + nestingLevel))])',
+            },
         pyatspi.ROLE_CANVAS: {
             'unfocused': '[Component(obj,\
                                      asString(((label + displayedText + imageDescription) or name) + roleName))]'
@@ -567,15 +626,13 @@ formatting = {
                                      asString(label + displayedText + roleName + availability) + asString(accelerator),\
                                      indicator=asString(checkedState))]'
             },
-        #pyatspi.ROLE_COLUMN_HEADER: 'default'
+        pyatspi.ROLE_COLUMN_HEADER: {
+            'unfocused': '[Component(obj,\
+                                     asString(((substring and currentLineText) or labelAndName) + roleName + sortOrder))]',
+            },
         pyatspi.ROLE_COMBO_BOX: {
-            # [[[TODO: WDW - maybe pass the label into the region constructor?
-            # We could then use the cursorOffset field to indicate where the
-            # combobox starts.]]]
-            #
-            'unfocused': '((comboBoxTextObj and ([Text(comboBoxTextObj[0], asString(label), asString(eol))] \
-                                               + [Region(" " + asString(roleName))])) \
-                           or [Component(obj, asString(label + name + roleName), label and (len(asString(label)) + 1) or 0)])'
+            'unfocused': '[Component(obj, asString(labelOrName + value + roleName), \
+                                     labelOrName and (len(asString(labelOrName)) + 1) or 0)]'
             },
         #pyatspi.ROLE_DESKTOP_ICON: 'default'
         pyatspi.ROLE_DIAL: {
@@ -583,7 +640,7 @@ formatting = {
                                      asString(labelOrName + value + roleName + required))]'
             },
         pyatspi.ROLE_DIALOG: {
-            'unfocused': '[Component(obj, asString(labelOrName + roleName))]'
+            'unfocused': '[Component(obj, asString(labelOrName + roleName + (unrelatedLabels or description)))]'
             },
         #pyatspi.ROLE_DIRECTORY_PANE: 'default'
         pyatspi.ROLE_DOCUMENT_FRAME: {
@@ -627,12 +684,10 @@ formatting = {
                                      asString(labelAndName + value + roleName + required))]',
             },
         pyatspi.ROLE_LABEL: {
-            'unfocused': '[Text(obj, asString(labelAndName))]'
+            'unfocused': BRAILLE_TEXT
             },
         pyatspi.ROLE_LINK: {
-            'unfocused': '[Link(obj, asString(currentLineText)\
-                                     or asString(displayedText)\
-                                     or asString(name))] \
+            'unfocused': '[Link(obj, asString(name or displayedText))] \
                         + (roleName and [Region(" " + asString(roleName))])',
         },
         pyatspi.ROLE_LIST: {
@@ -646,12 +701,12 @@ formatting = {
                                      asString(label) and (len(asString(label)) + 1) or 0)]'
         },
         pyatspi.ROLE_LIST_ITEM: {
-            'focused':   '((substring and ([Region(asString(listItemMarker))] + ' + BRAILLE_TEXT + '))\
+            'focused':   '((substring and ' + BRAILLE_TEXT + ')\
                           or ([Component(obj,\
-                                     asString(label + displayedText + expandableState + roleName + availability) + asString(accelerator))]\
+                                     asString(label + displayedText + expandableState + roleName + availability) + asString(accelerator), indicator=asString(checkedStateIfCheckable))] \
                           + (nestingLevel and [Region(" " + asString(nestingLevel))])\
                           + (listBoxItemWidgets and ([Region(" ")] + listBoxItemWidgets))))',
-            'unfocused': '((substring and ([Region(asString(listItemMarker))] + ' + BRAILLE_TEXT + '))\
+            'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
                           or ([Component(obj, asString(labelOrName + expandableState))]\
                               + (nestingLevel and [Region(" " + asString(nestingLevel))])\
                               + (listBoxItemWidgets and ([Region(" ")] + listBoxItemWidgets))))',
@@ -666,7 +721,11 @@ formatting = {
         pyatspi.ROLE_MENU_ITEM: {
             'unfocused': '[Component(obj,\
                                      asString(labelOrName + expandableState + availability) + asString(accelerator),\
-                                     indicator=asString(menuItemCheckedState))]'
+                                     indicator=asString(checkedStateIfCheckable))]'
+            },
+        pyatspi.ROLE_NOTIFICATION: {
+            'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
+                          or ([Component(obj, asString(labelAndName + roleName))]))'
             },
         pyatspi.ROLE_PAGE: {
             'unfocused': BRAILLE_TEXT
@@ -714,7 +773,10 @@ formatting = {
                                      + asString(accelerator),\
                                      indicator=asString(radioState))]'
             },
-        #pyatspi.ROLE_ROW_HEADER: 'default'
+        pyatspi.ROLE_ROW_HEADER: {
+            'unfocused': '[Component(obj,\
+                                     asString(((substring and currentLineText) or labelAndName) + roleName + sortOrder))]',
+            },
         pyatspi.ROLE_SCROLL_BAR: {
             'unfocused': '[Component(obj,\
                                      asString(labelOrName + value + roleName + required))]'
@@ -723,9 +785,7 @@ formatting = {
             'unfocused': 'asPageTabOrScrollPane'
             },
         pyatspi.ROLE_SECTION: {
-            'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
-                          or ([Component(obj, asString(labelAndName + roleName))]\
-                             + (childWidget and ([Region(" ")] + childWidget))))'
+            'unfocused': BRAILLE_TEXT
             },
         #'REAL_ROLE_SCROLL_PANE': 'default'
         pyatspi.ROLE_SLIDER: {
@@ -737,6 +797,10 @@ formatting = {
                           + (required and [Region(" " + asString(required))] or [])\
                           + (readOnly and [Region(" " + asString(readOnly))] or [])'
             },
+        pyatspi.ROLE_STATUS_BAR: {
+            'unfocused': '[Component(obj, asString(labelOrName + roleName))]\
+                              + [Region(" ")] + statusBar',
+            },
         'ROLE_SWITCH' : {
             'unfocused': '[Component(obj,\
                                      asString((labelOrName or description) + roleName),\
@@ -779,10 +843,13 @@ formatting = {
         pyatspi.ROLE_TOOL_BAR: {
             'unfocused': '[Component(obj, asString(labelOrName + roleName))]',
             },
+        pyatspi.ROLE_TREE: {
+            'unfocused': '[Component(obj, asString(labelOrName + roleName))]',
+            },
         pyatspi.ROLE_TREE_ITEM: {
             'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
                           or ([Component(obj, asString(labelOrName + expandableState))]\
-                              + (nestingLevel and [Region(" " + asString(nestingLevel))])))',
+                              + (nodeLevel and [Region(" " + asString(nodeLevel))])))',
             },
         #pyatspi.ROLE_TREE: 'default'
         #pyatspi.ROLE_TREE_TABLE: 'default'
@@ -994,6 +1061,12 @@ class Formatting(dict):
         except:
             pass
 
+        if args.get('formatType') == 'ancestor':
+            try:
+                return self[args['mode']][args['role']]['focused']
+            except:
+                pass
+
         if args.get('formatType') == 'detailedWhereAmI':
             try:
                 return self[args['mode']][args['role']]['basicWhereAmI']
