@@ -81,24 +81,25 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             if priorDoc != doc and not self._script.utilities.getDocumentForObject(doc):
                 result = [super()._generateName(doc)]
 
-        if self._script.utilities.isLink(obj) \
-           or self._script.utilities.isLandmark(obj) \
-           or self._script.utilities.isMath(obj):
+        if not self._script.utilities.getTable(obj) \
+           and (self._script.utilities.isLink(obj) \
+                or self._script.utilities.isLandmark(obj) \
+                or self._script.utilities.isMath(obj) \
+                or obj.getRole() in [pyatspi.ROLE_TOOL_TIP, pyatspi.ROLE_STATUS_BAR]):
             return result
 
-        args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_FRAME,
-                               pyatspi.ROLE_DOCUMENT_WEB,
+        args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_WEB,
                                pyatspi.ROLE_EMBEDDED,
                                pyatspi.ROLE_INTERNAL_FRAME,
                                pyatspi.ROLE_MATH,
-                               pyatspi.ROLE_MENU_BAR,
-                               pyatspi.ROLE_TOOL_BAR]
+                               pyatspi.ROLE_MENU_BAR]
         args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
                              pyatspi.ROLE_HEADING,
                              pyatspi.ROLE_LABEL,
                              pyatspi.ROLE_LINK,
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_TEXT]
+        args['stopAfterRoles'] = [pyatspi.ROLE_TOOL_BAR]
 
         if self._script.utilities.isEditableDescendantOfComboBox(obj):
             args['skipRoles'].append(pyatspi.ROLE_COMBO_BOX)
@@ -146,7 +147,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             result = [messages.HAS_POPUP_TREE]
 
         if result:
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         return result
 
@@ -163,7 +164,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         args['stringType'] = 'clickable'
         if self._script.utilities.isClickableElement(obj):
             result = [self._script.formatting.getString(**args)]
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
         return []
@@ -214,7 +215,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         args['stringType'] = 'haslongdesc'
         if self._script.utilities.hasLongDesc(obj):
             result = [self._script.formatting.getString(**args)]
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
         return []
@@ -235,7 +236,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         args['stringType'] = 'hasdetails'
         result = [self._script.formatting.getString(**args) % toPresent]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
     def _generateAllDetails(self, obj, **args):
@@ -252,19 +253,19 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         args['stringType'] = 'hasdetails'
         result = [self._script.formatting.getString(**args) % ""]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         result = []
         for o in objs:
             result.append(self.getLocalizedRoleName(o))
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
             string = self._script.utilities.expandEOCs(o)
             if not string.strip():
                 continue
 
             result.append(string)
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             result.extend(self._generatePause(o))
 
         return result
@@ -300,7 +301,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 words = words[0:5] + ['...']
 
             result.append(self._script.formatting.getString(**objArgs) % " ".join(words))
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             result.extend(self._generatePause(o, **objArgs))
 
         return result
@@ -311,18 +312,51 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         if self._script.utilities.isTextBlockElement(obj) \
            and not self._script.utilities.isLandmark(obj) \
+           and not self._script.utilities.isDocument(obj) \
            and not self._script.utilities.isDPub(obj) \
            and not self._script.utilities.isContentSuggestion(obj):
             return []
 
+        priorObj = args.get("priorObj")
+        if obj == priorObj:
+            return []
+
+        if priorObj and priorObj in self._script.utilities.labelsForObject(obj):
+            return []
+
+        descendant = args.get("ancestorOf")
+        if descendant and priorObj and obj.name and obj.name == priorObj.name:
+            msg = "WEB: %s's ancestor %s has same name as priorObj %s. Not generating labelOrName." \
+                % (descendant, obj, priorObj)
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return []
+
+        if self._script.utilities.isContentEditableWithEmbeddedObjects(obj) \
+           or self._script.utilities.isDocument(obj):
+            lastKey, mods = self._script.utilities.lastKeyAndModifiers()
+            if lastKey in ["Home", "End", "Up", "Down", "Left", "Right", "Page_Up", "Page_Down"]:
+                return []
+
+        if priorObj and priorObj.getRole() == pyatspi.ROLE_PAGE_TAB and priorObj.name == obj.name:
+            return []
+
         if obj.name:
             name = obj.name
             if not self._script.utilities.hasExplicitName(obj):
                 name = name.strip()
+
+            if self._script.utilities.shouldVerbalizeAllPunctuation(obj):
+                name = self._script.utilities.verbalizeAllPunctuation(name)
+
             result = [name]
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             return result
 
+        if obj.getRole() == pyatspi.ROLE_CHECK_BOX:
+            gridCell = pyatspi.findAncestor(obj, self._script.utilities.isGridCell)
+            if gridCell:
+                return super()._generateLabelOrName(gridCell, **args)
+
         return super()._generateLabelOrName(obj, **args)
 
     def _generateName(self, obj, **args):
@@ -331,16 +365,22 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         if self._script.utilities.isTextBlockElement(obj) \
            and not self._script.utilities.isLandmark(obj) \
-           and not self._script.utilities.isDPub(obj):
+           and not self._script.utilities.isDPub(obj) \
+           and not args.get('inFlatReview'):
             return []
 
+        if self._script.utilities.hasVisibleCaption(obj):
+            return []
+
+        if self._script.utilities.isFigure(obj) and args.get('ancestorOf'):
+            caption = args.get('ancestorOf')
+            if caption.getRole() != pyatspi.ROLE_CAPTION:
+                isCaption = lambda x: x and x.getRole() == pyatspi.ROLE_CAPTION
+                caption = pyatspi.findAncestor(caption, isCaption)
+            if caption and hash(obj) in self._script.utilities.labelTargets(caption):
+                return []
+
         role = args.get('role', obj.getRole())
-        alwaysPresent = [pyatspi.ROLE_PUSH_BUTTON,
-                         pyatspi.ROLE_IMAGE]
-
-        if obj.parent and obj.name and obj.name == obj.parent.name \
-           and obj != orca_state.locusOfFocus and role not in alwaysPresent:
-            return []
 
         # TODO - JD: Once the formatting strings are vastly cleaned up
         # or simply removed, hacks like this won't be needed.
@@ -359,7 +399,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     name = name.strip()
                 result = [name]
 
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             return result
 
         return super()._generateName(obj, **args)
@@ -374,7 +414,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         label, objects = self._script.utilities.inferLabelFor(obj)
         if label:
             result = [label]
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             return result
 
         return super()._generateLabel(obj, **args)
@@ -399,7 +439,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.inDocumentContent(obj) \
            and not self._script.utilities.inDocumentContent(orca_state.locusOfFocus):
             result = ['']
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
         return super()._generateLeaving(obj, **args)
@@ -419,19 +459,22 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         # other toolkits (e.g. exposing list items to us that are not
         # exposed to sighted users)
         role = args.get('role', obj.getRole())
-        if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
+        if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX, pyatspi.ROLE_DESCRIPTION_LIST]:
             return super()._generateNumberOfChildren(obj, **args)
 
         setsize = self._script.utilities.getSetSize(obj[0])
         if setsize is None:
-            children = [x for x in obj if x.getRole() == pyatspi.ROLE_LIST_ITEM]
+            if self._script.utilities.isDescriptionList(obj):
+                children = [x for x in obj if self._script.utilities.isDescriptionListTerm(x)]
+            else:
+                children = [x for x in obj if x.getRole() == pyatspi.ROLE_LIST_ITEM]
             setsize = len(children)
 
         if not setsize:
             return []
 
         result = [messages.listItemCount(setsize)]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
     # TODO - JD: Yet another dumb generator method we should kill.
@@ -461,13 +504,14 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not self._script.utilities.inDocumentContent(obj):
             return super()._generateRoleName(obj, **args)
 
-        result = []
-        acss = self.voice(speech_generator.SYSTEM)
+        if obj == args.get('priorObj'):
+            return []
 
+        result = []
         roledescription = self._script.utilities.getRoleDescription(obj)
         if roledescription:
             result = [roledescription]
-            result.extend(acss)
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
         role = args.get('role', obj.getRole())
@@ -525,13 +569,13 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     return []
             if role not in doNotSpeak:
                 result.append(self.getLocalizedRoleName(obj, **args))
-                result.extend(acss)
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         elif isEditable and self._script.utilities.isDocument(obj):
             if obj.parent and not obj.parent.getState().contains(pyatspi.STATE_EDITABLE) \
                and lastKey not in ["Home", "End", "Up", "Down", "Left", "Right", "Page_Up", "Page_Down"]:
                 result.append(object_properties.ROLE_EDITABLE_CONTENT)
-                result.extend(acss)
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         elif role == pyatspi.ROLE_HEADING:
             if index == total - 1 or not self._script.utilities.isFocusableWithMathChild(obj):
@@ -540,26 +584,26 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     result.append(object_properties.ROLE_HEADING_LEVEL_SPEECH % {
                         'role': self.getLocalizedRoleName(obj, **args),
                         'level': level})
-                    result.extend(acss)
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
                 else:
                     result.append(self.getLocalizedRoleName(obj, **args))
-                    result.extend(acss)
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         elif self._script.utilities.isLink(obj):
             if obj.parent.getRole() == pyatspi.ROLE_IMAGE:
                 result.append(messages.IMAGE_MAP_LINK)
-                result.extend(acss)
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             else:
                 if self._script.utilities.hasUselessCanvasDescendant(obj):
                     result.append(self.getLocalizedRoleName(obj, role=pyatspi.ROLE_IMAGE))
-                    result.extend(acss)
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
                 if index == total - 1 or not self._script.utilities.isFocusableWithMathChild(obj):
                     result.append(self.getLocalizedRoleName(obj, **args))
-                    result.extend(acss)
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         elif role not in doNotSpeak and args.get('priorObj') != obj:
             result.append(self.getLocalizedRoleName(obj, **args))
-            result.extend(acss)
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         if self._script.utilities.isMath(obj) and not self._script.utilities.isMathTopLevel(obj):
             return result
@@ -583,7 +627,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             return []
 
         result = [string]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
     def _generateSiteDescription(self, obj, **args):
@@ -617,7 +661,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result.append(messages.LINK_DIFFERENT_SITE)
 
         if result:
-            result.extend(self.voice(speech_generator.HYPERLINK))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         return result
 
@@ -655,6 +699,10 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.isEditableComboBox(obj):
             return []
 
+        if args.get('formatType') not in ['basicWhereAmI', 'detailedWhereAmI']:
+            if args.get('priorObj') == obj:
+                return []
+
         position = self._script.utilities.getPositionInSet(obj)
         total = self._script.utilities.getSetSize(obj)
         if position is None or total is None:
@@ -671,9 +719,15 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             stringType='groupindex') \
             % {"index" : position,
                "total" : total})
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
+    def _generateUnselectedCell(self, obj, **args):
+        if not self._script.inFocusMode():
+            return []
+
+        return super()._generateUnselectedCell(obj, **args)
+
     def _generateRealTableCell(self, obj, **args):
         result = super()._generateRealTableCell(obj, **args)
         if not self._script.inFocusMode():
@@ -683,16 +737,16 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             label = self._script.utilities.labelForCellCoordinates(obj)
             if label:
                 result.append(label)
-                result.extend(self.voice(speech_generator.SYSTEM))
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
                 return result
 
             row, col = self._script.utilities.coordinatesForCell(obj)
             if self._script.utilities.cellRowChanged(obj):
                 result.append(messages.TABLE_ROW % (row + 1))
-                result.extend(self.voice(speech_generator.SYSTEM))
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             if self._script.utilities.cellColumnChanged(obj):
                 result.append(messages.TABLE_COLUMN % (col + 1))
-                result.extend(self.voice(speech_generator.SYSTEM))
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         return result
 
@@ -711,9 +765,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return super()._generateTableCellRow(obj, **args)
 
     def _generateRowHeader(self, obj, **args):
-        # TODO - JD: _lastCommandWasCaretNav is private.
-        if self._script.utilities.lastInputEventWasLineNav() \
-           and self._script._lastCommandWasCaretNav:
+        if self._script.utilities.lastInputEventWasLineNav():
             return []
 
         return super()._generateRowHeader(obj)
@@ -732,26 +784,28 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             oldRole = self._overrideRole('default', args)
         elif self._script.utilities.isLink(obj):
             oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
+        elif self._script.utilities.isCustomImage(obj):
+            oldRole = self._overrideRole(pyatspi.ROLE_IMAGE, args)
         elif self._script.utilities.treatAsDiv(obj, offset=args.get('startOffset')):
             oldRole = self._overrideRole(pyatspi.ROLE_SECTION, args)
         else:
             oldRole = self._overrideRole(self._getAlternativeRole(obj, **args), args)
 
         if not 'priorObj' in args:
-            args['priorObj'] = self._script.utilities.getPriorContext()[0]
+            document = self._script.utilities.getTopLevelDocumentForObject(obj)
+            args['priorObj'] = self._script.utilities.getPriorContext(document)[0]
 
-        if self._script.utilities.isLabellingContents(obj):
-            result = list(filter(lambda x: x, self.generateContext(obj, **args)))
+        start = args.get("startOffset", 0)
+        end = args.get("endOffset", -1)
+        args["language"], args["dialect"] = \
+            self._script.utilities.getLanguageAndDialectForSubstring(obj, start, end)
 
         if not result:
             result = list(filter(lambda x: x, super().generateSpeech(obj, **args)))
 
         self._restoreRole(oldRole, args)
-        msg = "WEB: Speech generation for document object %s complete:" % obj
+        msg = "WEB: Speech generation for document object %s complete." % obj
         debug.println(debug.LEVEL_INFO, msg, True)
-        for element in result:
-            debug.println(debug.LEVEL_ALL, "%s%s" % (' ' * 18, element))
-
         return result
 
     def generateContents(self, contents, **args):
@@ -778,10 +832,12 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 args['priorObj'] = obj
 
         if not result:
-            if self._script.inSayAll():
+            if self._script.inSayAll(treatInterruptedAsIn=False) \
+               or not _settingsManager.getSetting('speakBlankLines') \
+               or args.get('formatType') == 'ancestor':
                 string = ""
             else:
                 string = messages.BLANK
-            result = [string, self.voice(speech_generator.DEFAULT)]
+            result = [string, self.voice(speech_generator.DEFAULT, **args)]
 
         return result
